
from googleapiclient.discovery import build
import json


class YoutubeAPI:
    def __init__(self, DEVELOPER_KEY: str):
        self._DEV_KEY = DEVELOPER_KEY
        self.YOUTUBE_API_SERVICE_NAME = "youtube"
        self.YOUTUBE_API_VERSION = "v3"

    def get_links(self, search_keyword: str, max_link_len: int) -> list:
        youtube = build(self.YOUTUBE_API_SERVICE_NAME, self.YOUTUBE_API_VERSION, developerKey=self._DEV_KEY)
        search_response = youtube.search().list(
            q=search_keyword,
            type='video',
            part='id,snippet',
            maxResults=max_link_len
        ).execute()

        links = []
        for search_result in search_response.get("items", []):
            if search_result["id"]["kind"] == "youtube#video":
                links.append("https://www.youtube.com/watch?v={}".format(search_result["id"]["videoId"]))
        
        return links


import time
import pandas as pd
from selenium.webdriver import Chrome
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC


class YoutubeCrawler:
    def __init__(self, chromedriver_path: str):
        self._chrome = chromedriver_path

    @staticmethod
    def get_comment(chromedriver_path: str, link: str, max_comment_pg_len: int)  -> list:
        data=[]
        with Chrome(executable_path=chromedriver_path) as driver:
            wait = WebDriverWait(driver, 15)
            driver.get(link)
            for item in range(max_comment_pg_len): 
                wait.until(EC.visibility_of_element_located((By.TAG_NAME, "body"))).send_keys(Keys.END)
                time.sleep(15)
            for comment in wait.until(EC.presence_of_all_elements_located((By.CSS_SELECTOR, "#content"))):
                data.append(comment.text)

        return data
    
    def get_comment(self, link: str, max_comment_pg_len: int) -> list:
        data=[]
        with Chrome(executable_path=self._chrome) as driver:
            wait = WebDriverWait(driver, 15)
            driver.get(link)
            for item in range(max_comment_pg_len): 
                wait.until(EC.visibility_of_element_located((By.TAG_NAME, "body"))).send_keys(Keys.END)
                time.sleep(15)
            for comment in wait.until(EC.presence_of_all_elements_located((By.CSS_SELECTOR, "#content"))):
                data.append(comment.text)

        return data

    def get_comment_df(self, links: list, max_comment_pg_len: int) -> pd.DataFrame:
        total_dict = dict()
        for i in range(len(links)):
            data = self.get_comment(links[i], max_comment_pg_len)
            if i==0:
                df = pd.DataFrame(data, columns=['comments'])
                df['link'] = links[0]
            else:
                temp_df = pd.DataFrame(data, columns=['comments'])
                temp_df['link'] = links[i]
                df = pd.concat([df, temp_df])
        df.dropna(how='any', inplace=True)
        
        return df        